@@ -39,14 +39,18 @@ GRN_API grn_bool grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_bulk(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_text_family_bulk(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_number_family_bulk(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool grn_obj_is_vector(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_vector(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_weight_vector(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_uvector(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_weight_uvector(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_table(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_lexicon(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_weight_vector_column(grn_ctx *ctx, grn_obj *obj);
-GRN_API grn_bool grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_data_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_index_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_accessor(grn_ctx *ctx, grn_obj *obj);
@@ -54,11 +58,13 @@ GRN_API grn_bool grn_obj_is_id_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_key_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_value_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_score_accessor(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_referable_score_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_nsubrecs_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_max_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_min_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_sum_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_avg_accessor(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_mean_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_column_value_accessor(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_type(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_text_family_type(grn_ctx *ctx, grn_obj *obj);
@@ -71,11 +77,14 @@ GRN_API grn_bool grn_obj_is_normalizer_proc(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_token_filter_proc(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_scorer_proc(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_window_function_proc(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_aggregator_proc(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_bool grn_obj_is_expr(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool grn_obj_is_visible(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc grn_obj_set_visibility(grn_ctx *ctx,
                                       grn_obj *obj,
                                       bool is_visible);
+GRN_API bool grn_obj_have_source(grn_ctx *ctx, grn_obj *obj);
+GRN_API bool grn_obj_is_token_column(grn_ctx *ctx, grn_obj *obj);
 
 GRN_API grn_rc grn_obj_cast(grn_ctx *ctx,
                             grn_obj *src,
