@@ -596,6 +596,10 @@ grn_accessor_execute(grn_ctx *ctx,
     grn_obj_close(ctx, base_res);
   }
 
+  if (n_index_datum > 0) {
+    grn_obj_unref(ctx, index_data.index);
+  }
+
   GRN_API_RETURN(rc);
 }
 
@@ -604,6 +608,7 @@ typedef struct {
   grn_obj *query;
   grn_search_optarg *optarg;
   grn_obj *index;
+  bool index_need_unref;
   grn_operator operator;
   grn_obj query_casted;
   const char *query_raw;
@@ -618,6 +623,9 @@ grn_accessor_estimate_size_for_query_data_fin(
   grn_ctx *ctx,
   grn_accessor_estimate_size_for_query_data *data)
 {
+  if (data->index_need_unref) {
+    grn_obj_unref(ctx, data->index);
+  }
   GRN_OBJ_FIN(ctx, &(data->query_casted));
 }
 
@@ -695,15 +703,16 @@ grn_accessor_estimate_size_for_query_prepare(
     data->index = a->obj;
   } else {
     data->index = index_data.index;
+    data->index_need_unref = true;
   }
 
   grn_obj *lexicon;
-  bool lexicon_need_unlink = false;
+  bool lexicon_need_unref = false;
   if (grn_obj_is_table(ctx, data->index)) {
     lexicon = data->index;
   } else {
     lexicon = grn_ctx_at(ctx, data->index->header.domain);
-    lexicon_need_unlink = true;
+    lexicon_need_unref = true;
   }
 
   if (data->query->header.domain == lexicon->header.domain) {
@@ -720,8 +729,8 @@ grn_accessor_estimate_size_for_query_prepare(
     }
   }
 
-  if (lexicon_need_unlink) {
-    grn_obj_unlink(ctx, lexicon);
+  if (lexicon_need_unref) {
+    grn_obj_unref(ctx, lexicon);
   }
 
   return ctx->rc == GRN_SUCCESS;
@@ -756,6 +765,7 @@ grn_accessor_estimate_size_for_query(grn_ctx *ctx,
   data.query = query;
   data.optarg = optarg;
   data.index = NULL;
+  data.index_need_unref = false;
   data.operator = GRN_OP_MATCH;
   GRN_VOID_INIT(&(data.query_casted));
   data.query_raw = NULL;
@@ -894,6 +904,12 @@ grn_accessor_name(grn_ctx *ctx, grn_obj *accessor, grn_obj *name)
                    GRN_COLUMN_NAME_AVG,
                    GRN_COLUMN_NAME_AVG_LEN);
       break;
+    case GRN_ACCESSOR_GET_MEAN :
+      GRN_TEXT_PUT(ctx,
+                   name,
+                   GRN_COLUMN_NAME_MEAN,
+                   GRN_COLUMN_NAME_MEAN_LEN);
+      break;
     case GRN_ACCESSOR_GET_COLUMN_VALUE :
       grn_column_name_(ctx, accessor_->obj, name);
       show_obj_domain_name = GRN_TRUE;
@@ -932,8 +948,8 @@ grn_accessor_name(grn_ctx *ctx, grn_obj *accessor, grn_obj *name)
         GRN_TEXT_PUT(ctx, name, target_name, target_name_size);
       }
       GRN_TEXT_PUTS(ctx, name, ")");
-      if (show_obj_domain_name && target && grn_enable_reference_count) {
-        grn_obj_unlink(ctx, target);
+      if (show_obj_domain_name && target) {
+        grn_obj_unref(ctx, target);
       }
     }
   }
