@@ -58,12 +58,6 @@ namespace grn {
 
     struct TableHandler : public rapidjson::BaseReaderHandler<>
     {
-      grn_ctx *ctx_;
-      grn_obj *uvector_;
-      bool add_record_if_not_exist_;
-      grn_obj *table_;
-      uint32_t weight_;
-
       TableHandler(grn_ctx *ctx,
                    grn_obj *uvector,
                    bool add_record_if_not_exist)
@@ -71,7 +65,7 @@ namespace grn {
           uvector_(uvector),
           add_record_if_not_exist_(add_record_if_not_exist),
           table_(grn_ctx_at(ctx, uvector->header.domain)),
-          weight_(0) {
+          weight_(0.0) {
       }
 
       ~TableHandler() {
@@ -90,7 +84,7 @@ namespace grn {
         if (id == GRN_ID_NIL) {
           return false;
         }
-        auto rc = grn_uvector_add_element(ctx_, uvector_, id, weight_);
+        auto rc = grn_uvector_add_element_record(ctx_, uvector_, id, weight_);
         return rc == GRN_SUCCESS;
       }
 
@@ -103,6 +97,18 @@ namespace grn {
         weight_ = value;
         return true;
       }
+
+      bool Double(double value) {
+        weight_ = value;
+        return true;
+      }
+
+    private:
+      grn_ctx *ctx_;
+      grn_obj *uvector_;
+      bool add_record_if_not_exist_;
+      grn_obj *table_;
+      float weight_;
     };
 
     template <typename Handler>
@@ -155,7 +161,20 @@ namespace grn {
     rapidjson::MemoryStream stream(GRN_TEXT_VALUE(src), GRN_TEXT_LEN(src));
     document.ParseStream(stream);
     if (document.HasParseError()) {
-      return GRN_INVALID_ARGUMENT;
+      auto domain = grn_ctx_at(ctx, dest->header.domain);
+      grn_rc rc = GRN_INVALID_ARGUMENT;
+      if (grn_obj_is_lexicon(ctx, domain)) {
+        grn_obj dest_record;
+        GRN_RECORD_INIT(&dest_record, GRN_BULK, dest->header.domain);
+        rc = grn_obj_cast(ctx, src, &dest_record, add_record_if_not_exist);
+        if (rc == GRN_SUCCESS && GRN_BULK_VSIZE(&dest_record) > 0) {
+          auto id = GRN_RECORD_VALUE(&dest_record);
+          GRN_RECORD_PUT(ctx, dest, id);
+        }
+        GRN_OBJ_FIN(ctx, &dest_record);
+      }
+      grn_obj_unref(ctx, domain);
+      return rc;
     }
     switch (dest->header.domain) {
     case GRN_DB_INT32 :
@@ -166,7 +185,7 @@ namespace grn {
     default :
       {
         grn_rc rc = GRN_INVALID_ARGUMENT;
-        grn_obj *domain = grn_ctx_at(ctx, dest->header.domain);
+        auto domain = grn_ctx_at(ctx, dest->header.domain);
         if (grn_obj_is_lexicon(ctx, domain)) {
           rc = json_to_uvector<TableHandler>(ctx,
                                              &document,
