@@ -38,6 +38,7 @@
 #include "grn_logger.h"
 #include "grn_cache.h"
 #include "grn_expr.h"
+#include "grn_token_column.h"
 #include "grn_windows.h"
 #include <stdio.h>
 #include <stdarg.h>
@@ -120,6 +121,7 @@ grn_init_from_env(void)
   grn_output_init_from_env();
   grn_proc_init_from_env();
   grn_plugin_init_from_env();
+  grn_token_column_init_from_env();
 }
 
 static void
