@@ -20,10 +20,11 @@
 #pragma once
 
 #include "grn.h"
+
 #include "grn_ctx.h"
 #include "grn_options.h"
-#include "grn_store.h"
 #include "grn_rset.h"
+#include "grn_store.h"
 
 #include <groonga/command.h>
 #include <groonga/token_filter.h>
@@ -56,6 +57,8 @@ extern "C" {
   (GRN_TABLE_IS_GROUPED(table) &&\
    table->header.domain == GRN_ID_NIL)
 
+#define DB_OBJ(obj) ((grn_db_obj *)obj)
+
 extern bool grn_enable_reference_count;
 
 typedef struct _grn_db grn_db;
@@ -201,8 +204,26 @@ int grn_table_get_key2(grn_ctx *ctx, grn_obj *table, grn_id id, grn_obj *bulk);
 grn_table_cursor *grn_table_cursor_open_by_id(grn_ctx *ctx, grn_obj *table,
                                               grn_id min, grn_id max, int flags);
 
-void grn_table_add_subrec(grn_obj *table, grn_rset_recinfo *ri, double score,
-                          grn_rset_posinfo *pi, int dir);
+grn_inline static void
+grn_table_add_subrec(grn_ctx *ctx,
+                     grn_obj *table,
+                     grn_rset_recinfo *ri,
+                     double score,
+                     grn_rset_posinfo *pi,
+                     int dir)
+{
+  if (DB_OBJ(table)->header.flags & GRN_OBJ_WITH_SUBREC) {
+    ri->score += score;
+    ri->n_subrecs += 1;
+    /* This is a duplicated check but it reduces the number of
+     * function calls. It improves performance when many records are
+     * matched. */
+    int limit = DB_OBJ(table)->max_n_subrecs;
+    if (limit > 0) {
+      grn_rset_add_subrec(ctx, ri, table, score, pi, dir);
+    }
+  }
+}
 
 grn_obj *grn_obj_graft(grn_ctx *ctx, grn_obj *obj);
 
@@ -319,6 +340,11 @@ struct _grn_proc {
       grn_scorer_score_func *score;
     } scorer;
     grn_window_function_func *window_function;
+    struct {
+      grn_aggregator_init_func *init;
+      grn_aggregator_update_func *update;
+      grn_aggregator_fin_func *fin;
+    } aggregator;
   } callbacks;
 
   void *user_data;
@@ -379,14 +405,13 @@ typedef enum {
   GRN_ACCESSOR_GET_MIN,
   GRN_ACCESSOR_GET_SUM,
   GRN_ACCESSOR_GET_AVG,
+  GRN_ACCESSOR_GET_MEAN,
   GRN_ACCESSOR_GET_COLUMN_VALUE,
   GRN_ACCESSOR_GET_DB_OBJ,
   GRN_ACCESSOR_LOOKUP,
   GRN_ACCESSOR_FUNCALL
 } grn_accessor_action;
 
-#define DB_OBJ(obj) ((grn_db_obj *)obj)
-
 GRN_API const char *grn_obj_get_value_(grn_ctx *ctx, grn_obj *obj, grn_id id, uint32_t *size);
 void grn_obj_get_range_info(grn_ctx *ctx,
                             grn_obj *obj,
@@ -467,7 +492,6 @@ grn_rc grn_db_obj_init(grn_ctx *ctx, grn_obj *db, grn_id id, grn_db_obj *obj);
 #define GRN_ACCESSORP(obj) \
   ((obj) && (((grn_obj *)(obj))->header.type == GRN_ACCESSOR))
 
-grn_rc grn_obj_refer(grn_ctx *ctx, grn_obj *obj);
 uint32_t grn_obj_reference_count(grn_ctx *ctx, grn_obj *obj);
 
 grn_id grn_obj_register(grn_ctx *ctx, grn_obj *db, const char *name, unsigned int name_size);
@@ -489,6 +513,16 @@ GRN_API grn_rc grn_table_group_with_range_gap(grn_ctx *ctx, grn_obj *table,
                                               grn_obj *result_set,
                                               uint32_t range_gap);
 
+grn_obj *grn_table_column(grn_ctx *ctx,
+                          grn_obj *table,
+                          const char *name,
+                          ssize_t name_size);
+grn_rc grn_table_parse_load_columns(grn_ctx *ctx,
+                                    grn_obj *table,
+                                    const char *input,
+                                    size_t input_size,
+                                    grn_obj *columns);
+
 GRN_API grn_rc grn_column_filter(grn_ctx *ctx, grn_obj *column,
                                  grn_operator op,
                                  grn_obj *value, grn_obj *result_set,
