@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2013-2018 Brazil
-  Copyright(C) 2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2019-2020 Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,7 @@
 #include "mrb_options.h"
 #include "mrb_converter.h"
 
+#include "../grn_db.h"
 #include "../grn_encoding.h"
 
 static mrb_value
@@ -78,6 +79,34 @@ grn_mrb_object_inspect(mrb_state *mrb, mrb_value self)
   return inspected;
 }
 
+static mrb_value
+object_refer(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  if (grn_enable_reference_count) {
+    grn_obj_refer(ctx, DATA_PTR(self));
+    grn_mrb_ctx_check(mrb);
+  }
+  return mrb_nil_value();
+}
+
+static mrb_value
+object_unref(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  grn_obj_unref(ctx, DATA_PTR(self));
+  grn_mrb_ctx_check(mrb);
+  return mrb_nil_value();
+}
+
+static mrb_value
+object_reference_count(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  uint32_t reference_count = grn_obj_reference_count(ctx, DATA_PTR(self));
+  return mrb_fixnum_value(reference_count);
+}
+
 static mrb_value
 object_get_id(mrb_state *mrb, mrb_value self)
 {
@@ -325,6 +354,11 @@ grn_mrb_object_init(grn_ctx *ctx)
   mrb_define_method(mrb, klass, "inspect",
                     grn_mrb_object_inspect, MRB_ARGS_NONE());
 
+  mrb_define_method(mrb, klass, "refer", object_refer, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "unref", object_unref, MRB_ARGS_NONE());
+  mrb_define_method(mrb, klass, "reference_count",
+                    object_reference_count, MRB_ARGS_NONE());
+
   mrb_define_method(mrb, klass, "id", object_get_id, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "name", object_get_name, MRB_ARGS_NONE());
   mrb_define_method(mrb, klass, "path", object_get_path, MRB_ARGS_NONE());
