@@ -53,9 +53,16 @@ module Groonga
       end
     end
 
+    def unref
+      @search_indexes.each(&:unref)
+    end
+
     private
     def near_search?
-      (@op == Operator::NEAR or @op == Operator::NEAR2) and @args.size == 3
+      (@op == Operator::NEAR or
+       @op == Operator::NEAR2 or
+       @op == Operator::NEAR_PHRASE) and
+        @args.size == 3
     end
 
     def match_near_resolve_index
@@ -69,7 +76,8 @@ module Groonga
         match_resolve_index_indexable(arg)
       else
         message =
-          "The first argument of NEAR/NEAR2 must be Expression, Accessor or Indexable: #{arg.class}"
+          "The first argument of NEAR/NEAR2/NEAR_PHRASE must be " +
+          "Expression, Accessor or Indexable: #{arg.class}"
         raise ErrorMessage, message
       end
 
@@ -291,6 +299,8 @@ module Groonga
         index_info = accessor.find_index(@op)
         if index_info
           if accessor.have_next? and index_info.index != accessor.object
+            accessor.refer
+            index_info.index.unref
             index_info = IndexInfo.new(accessor, index_info.section_id)
           end
         end
@@ -302,12 +312,13 @@ module Groonga
         rest_n_codes = n_codes - i
         if rest_n_codes >= 2 and
           codes[i + 1].value.is_a?(Bulk) and
-          (codes[i + 1].value.domain == ID::UINT32 or
-           codes[i + 1].value.domain == ID::INT32) and
+          (codes[i + 1].value.domain_id == ID::UINT32 or
+           codes[i + 1].value.domain_id == ID::INT32) and
           codes[i + 2].op == Operator::GET_MEMBER
           section_id = codes[i + 1].value.value + 1
           offset += 2
         end
+        index.refer
         index_info = IndexInfo.new(index, section_id)
       end
 
@@ -323,6 +334,8 @@ module Groonga
       section_id = index_info.section_id
       weight = expr_code.weight
       if accessor.next
+        accessor.refer
+        index_info.index.unref
         put_search_index(accessor, section_id, weight)
       else
         put_search_index(index_info.index, section_id, weight)
@@ -337,6 +350,7 @@ module Groonga
     end
 
     def match_resolve_index_index_column(index)
+      index.refer
       put_search_index(index, 0, 1)
     end
 
@@ -351,6 +365,8 @@ module Groonga
       index_info = accessor.find_index(op)
       return if index_info.nil?
       if accessor.next
+        accessor.refer
+        index_info.index.unref
         put_search_index(accessor, index_info.section_id, 1)
       else
         put_search_index(index_info.index, index_info.section_id, 1)
