@@ -255,6 +255,16 @@ grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj)
   return (obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) == GRN_OBJ_COLUMN_SCALAR;
 }
 
+bool
+grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_scalar_column(ctx, obj)) {
+    return false;
+  }
+
+  return grn_type_id_is_number_family(ctx, grn_obj_get_range(ctx, obj));
+}
+
 grn_bool
 grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj)
 {
@@ -407,6 +417,31 @@ grn_obj_is_score_accessor(grn_ctx *ctx, grn_obj *obj)
   return accessor->action == GRN_ACCESSOR_GET_SCORE;
 }
 
+bool
+grn_obj_is_referable_score_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_score_accessor(ctx, obj)) {
+    return false;
+  }
+
+  grn_accessor *accessor = (grn_accessor *)obj;
+  grn_id parent_domain = accessor->obj->header.domain;
+  if (!(parent_domain & GRN_OBJ_TMP_OBJECT)) {
+    return true;
+  }
+
+  bool need_recursive_collect = false;
+  grn_obj *parent = grn_ctx_at(ctx, parent_domain);
+  if (parent) {
+    if (parent->header.flags & GRN_OBJ_WITH_SUBREC) {
+      need_recursive_collect = true;
+    }
+    grn_obj_unref(ctx, parent);
+  }
+
+  return !need_recursive_collect;
+}
+
 bool
 grn_obj_is_nsubrecs_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -492,6 +527,23 @@ grn_obj_is_avg_accessor(grn_ctx *ctx, grn_obj *obj)
   return accessor->action == GRN_ACCESSOR_GET_AVG;
 }
 
+bool
+grn_obj_is_mean_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  if (accessor->next) {
+    return false;
+  }
+
+  return accessor->action == GRN_ACCESSOR_GET_MEAN;
+}
+
 bool
 grn_obj_is_column_value_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -643,6 +695,19 @@ grn_obj_is_window_function_proc(grn_ctx *ctx, grn_obj *obj)
   return proc->type == GRN_PROC_WINDOW_FUNCTION;
 }
 
+bool
+grn_obj_is_aggregator_proc(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_proc *proc;
+
+  if (!grn_obj_is_proc(ctx, obj)) {
+    return false;
+  }
+
+  proc = (grn_proc *)obj;
+  return proc->type == GRN_PROC_AGGREGATOR;
+}
+
 grn_bool
 grn_obj_is_expr(grn_ctx *ctx, grn_obj *obj)
 {
