@@ -127,7 +127,7 @@ command_object_list(grn_ctx *ctx,
     grn_ctx_output_str(ctx, name, name_size);
 
     GRN_BULK_REWIND(&vector);
-    grn_rc rc = grn_vector_unpack(ctx, &vector, value, value_len, false);
+    grn_rc rc = grn_vector_unpack(ctx, &vector, value, value_len, 0);
     if (rc != GRN_SUCCESS) {
       grn_ctx_output_map_open(ctx, "object", 4);
       {
