@@ -94,6 +94,7 @@ snippet_exec(grn_ctx *ctx, grn_obj *snip, grn_obj *text,
 static grn_obj *
 func_snippet(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
 {
+  const char *tag = "[snippet]";
   grn_obj *snippets = NULL;
   grn_obj *snip = NULL;
 
@@ -102,104 +103,98 @@ func_snippet(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
   if (nargs > N_REQUIRED_ARGS) {
     grn_obj *text = args[0];
     grn_obj *end_arg = args[nargs - 1];
-    unsigned int width = 200;
-    unsigned int max_n_results = 3;
+    int64_t width = 200;
+    int64_t max_n_results = 3;
     grn_snip_mapping *mapping = NULL;
-    int flags = GRN_SNIP_SKIP_LEADING_SPACES;
-    const char *prefix = NULL;
-    int prefix_length = 0;
-    const char *suffix = NULL;
-    int suffix_length = 0;
-    const char *normalizer_name = NULL;
-    int normalizer_name_length = 0;
-    const char *default_open_tag = NULL;
-    int default_open_tag_length = 0;
-    const char *default_close_tag = NULL;
-    int default_close_tag_length = 0;
+    int flags = 0;
+    bool skip_leading_spaces = true;
+    bool html_escape = false;
+    grn_obj *prefix = NULL;
+    grn_obj *suffix = NULL;
+    grn_obj *normalizer_name = NULL;
+    grn_obj *default_open_tag = NULL;
+    grn_obj *default_close_tag = NULL;
     int n_args_without_option = nargs;
     grn_obj *default_return_value = NULL;
 
     if (end_arg->header.type == GRN_TABLE_HASH_KEY) {
       grn_obj *options = end_arg;
-      grn_hash_cursor *cursor;
-      void *key;
-      int key_size;
-      grn_obj *value;
-
       n_args_without_option--;
-      cursor = grn_hash_cursor_open(ctx, (grn_hash *)options,
-                                    NULL, 0, NULL, 0,
-                                    0, -1, 0);
-      if (!cursor) {
-        GRN_PLUGIN_ERROR(ctx, GRN_NO_MEMORY_AVAILABLE,
-                         "snippet(): couldn't open cursor");
+      grn_rc rc = grn_proc_options_parse(ctx,
+                                         options,
+                                         tag,
+                                         "width",
+                                         GRN_PROC_OPTION_VALUE_INT64,
+                                         &width,
+                                         "max_n_results",
+                                         GRN_PROC_OPTION_VALUE_INT64,
+                                         &max_n_results,
+                                         "skip_leading_spaces",
+                                         GRN_PROC_OPTION_VALUE_BOOL,
+                                         &skip_leading_spaces,
+                                         "html_escape",
+                                         GRN_PROC_OPTION_VALUE_BOOL,
+                                         &html_escape,
+                                         "prefix",
+                                         GRN_PROC_OPTION_VALUE_RAW,
+                                         &prefix,
+                                         "suffix",
+                                         GRN_PROC_OPTION_VALUE_RAW,
+                                         &suffix,
+                                         "normalizer",
+                                         GRN_PROC_OPTION_VALUE_RAW,
+                                         &normalizer_name,
+                                         "default_open_tag",
+                                         GRN_PROC_OPTION_VALUE_RAW,
+                                         &default_open_tag,
+                                         "default_close_tag",
+                                         GRN_PROC_OPTION_VALUE_RAW,
+                                         &default_close_tag,
+                                         "default",
+                                         GRN_PROC_OPTION_VALUE_RAW,
+                                         &default_return_value,
+                                         NULL);
+      if (rc != GRN_SUCCESS) {
         goto exit;
       }
-      while (grn_hash_cursor_next(ctx, cursor) != GRN_ID_NIL) {
-        grn_hash_cursor_get_key_value(ctx, cursor,
-                                      &key, &key_size,
-                                      (void **)&value);
-        if (key_size == 5 && !memcmp(key, "width", 5)) {
-          width = GRN_UINT32_VALUE(value);
-        } else if (key_size == 13 && !memcmp(key, "max_n_results", 13)) {
-          max_n_results = GRN_UINT32_VALUE(value);
-        } else if (key_size == 19 && !memcmp(key, "skip_leading_spaces", 19)) {
-          if (GRN_BOOL_VALUE(value) == GRN_FALSE) {
-            flags &= ~GRN_SNIP_SKIP_LEADING_SPACES;
-          }
-        } else if (key_size == 11 && !memcmp(key, "html_escape", 11)) {
-          if (GRN_BOOL_VALUE(value)) {
-            mapping = GRN_SNIP_MAPPING_HTML_ESCAPE;
-          }
-        } else if (key_size == 6 && !memcmp(key, "prefix", 6)) {
-          prefix = GRN_TEXT_VALUE(value);
-          prefix_length = GRN_TEXT_LEN(value);
-        } else if (key_size == 6 && !memcmp(key, "suffix", 6)) {
-          suffix = GRN_TEXT_VALUE(value);
-          suffix_length = GRN_TEXT_LEN(value);
-        } else if (key_size == 10 && !memcmp(key, "normalizer", 10)) {
-          normalizer_name = GRN_TEXT_VALUE(value);
-          normalizer_name_length = GRN_TEXT_LEN(value);
-        } else if (key_size == 16 && !memcmp(key, "default_open_tag", 16)) {
-          default_open_tag = GRN_TEXT_VALUE(value);
-          default_open_tag_length = GRN_TEXT_LEN(value);
-        } else if (key_size == 17 && !memcmp(key, "default_close_tag", 17)) {
-          default_close_tag = GRN_TEXT_VALUE(value);
-          default_close_tag_length = GRN_TEXT_LEN(value);
-        } else if (key_size == 7 && !memcmp(key, "default", 7)) {
-          if (value->header.type == GRN_PTR) {
-            default_return_value = GRN_PTR_VALUE(value);
-          } else {
-            default_return_value = value;
-          }
-        } else {
-          GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
-                           "invalid option name: <%.*s>",
-                           key_size, (char *)key);
-          grn_hash_cursor_close(ctx, cursor);
-          goto exit;
-        }
-      }
-      grn_hash_cursor_close(ctx, cursor);
     }
 
-    snip = grn_snip_open(ctx, flags, width, max_n_results,
-                         default_open_tag, default_open_tag_length,
-                         default_close_tag, default_close_tag_length, mapping);
+    if (skip_leading_spaces) {
+      flags |= GRN_SNIP_SKIP_LEADING_SPACES;
+    }
+    if (html_escape) {
+      mapping = GRN_SNIP_MAPPING_HTML_ESCAPE;
+    }
+    if (default_return_value && default_return_value->header.type == GRN_PTR) {
+      default_return_value = GRN_PTR_VALUE(default_return_value);
+    }
+
+    snip = grn_snip_open(ctx,
+                         flags,
+                         width,
+                         max_n_results,
+                         default_open_tag ? GRN_TEXT_VALUE(default_open_tag) : NULL,
+                         default_open_tag ? GRN_TEXT_LEN(default_open_tag) : 0,
+                         default_close_tag ? GRN_TEXT_VALUE(default_close_tag) : NULL,
+                         default_close_tag ? GRN_TEXT_LEN(default_close_tag) : 0,
+                         mapping);
     if (snip) {
       grn_rc rc;
       unsigned int i;
       if (!normalizer_name) {
         grn_snip_set_normalizer(ctx, snip, GRN_NORMALIZER_AUTO);
-      } else if (normalizer_name_length > 0) {
+      } else if (GRN_TEXT_LEN(normalizer_name) > 0) {
         grn_obj *normalizer;
-        normalizer = grn_ctx_get(ctx, normalizer_name, normalizer_name_length);
+        normalizer = grn_ctx_get(ctx,
+                                 GRN_TEXT_VALUE(normalizer_name),
+                                 GRN_TEXT_LEN(normalizer_name));
         if (!grn_obj_is_normalizer_proc(ctx, normalizer)) {
           grn_obj inspected;
           GRN_TEXT_INIT(&inspected, 0);
           grn_inspect(ctx, &inspected, normalizer);
           GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
-                           "snippet(): not normalizer: <%.*s>",
+                           "%s not normalizer: <%.*s>",
+                           tag,
                            (int)GRN_TEXT_LEN(&inspected),
                            GRN_TEXT_VALUE(&inspected));
           GRN_OBJ_FIN(ctx, &inspected);
@@ -209,7 +204,8 @@ func_snippet(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
         grn_snip_set_normalizer(ctx, snip, normalizer);
         grn_obj_unlink(ctx, normalizer);
       }
-      if (default_open_tag_length == 0 && default_close_tag_length == 0) {
+      if ((!default_open_tag || GRN_TEXT_LEN(default_open_tag) == 0) &&
+          (!default_close_tag || GRN_TEXT_LEN(default_close_tag) == 0)) {
         unsigned int n_keyword_sets =
           (n_args_without_option - N_REQUIRED_ARGS) / KEYWORD_SET_SIZE;
         grn_obj **keyword_set_args = args + N_REQUIRED_ARGS;
@@ -233,11 +229,15 @@ func_snippet(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
                                  NULL, 0);
         }
       }
-      snippets = snippet_exec(ctx, snip, text,
+      snippets = snippet_exec(ctx,
+                              snip,
+                              text,
                               default_return_value,
                               user_data,
-                              prefix, prefix_length,
-                              suffix, suffix_length);
+                              prefix ? GRN_TEXT_VALUE(prefix) : NULL,
+                              prefix ? GRN_TEXT_LEN(prefix) : 0,
+                              suffix ? GRN_TEXT_VALUE(suffix) : NULL,
+                              suffix ? GRN_TEXT_LEN(suffix) : 0);
     }
   }
 #undef KEYWORD_SET_SIZE
