@@ -86,6 +86,6 @@ class PackagesGroongaOrgPackageTask < PackageTask
   end
 
   def dput_incoming
-    ENV["DPUT_INCOMING"] || "~groonga/ubuntu/nightly"
+    ENV["DPUT_INCOMING"] || "~groonga/ppa/ubuntu/"
   end
 end
