@@ -10,6 +10,7 @@ module Groonga
         @command_name = command_name
         @input = input
         @options = options
+        @shards = []
         initialize_parameters
       end
 
@@ -21,6 +22,10 @@ module Groonga
         each_internal(:descending, &block)
       end
 
+      def unref
+        @shards.each(&:unref)
+      end
+
       private
       def each_internal(order)
         context = Context.instance
@@ -77,7 +82,9 @@ module Groonga
             next
           end
 
-          shards << Shard.new(name, @shard_key_name, shard_range_data)
+          shard = Shard.new(name, @shard_key_name, shard_range_data)
+          shards << shard
+          @shards << shard
           next if shards.size < 3
           yield(*shards)
           shards.shift
@@ -88,7 +95,6 @@ module Groonga
         end
       end
 
-      private
       def initialize_parameters
         @logical_table = @input[:logical_table]
         if @logical_table.nil?
@@ -121,6 +127,8 @@ module Groonga
           @table_name = table_name
           @key_name = key_name
           @range_data = range_data
+          @table = nil
+          @key = nil
         end
 
         def table
@@ -134,6 +142,11 @@ module Groonga
         def key
           @key ||= Context.instance[full_key_name]
         end
+
+        def unref
+          @table.unref if @table
+          @key.unref if @key
+        end
       end
 
       class ShardRangeData
@@ -146,9 +159,9 @@ module Groonga
 
         def to_suffix
           if @day.nil?
-            "_%02d%02d" % [@year, @month]
+            "_%04d%02d" % [@year, @month]
           else
-            "_%02d%02d%02d" % [@year, @month, @day]
+            "_%04d%02d%02d" % [@year, @month, @day]
           end
         end
       end
@@ -162,7 +175,11 @@ module Groonga
         end
 
         def least_over_time
-          Time.local(@year, @month, @day + 1)
+          next_day = Time.local(@year, @month, @day) + (60 * 60 * 24)
+          while next_day.day == @day # For leap second
+            next_day += 1
+          end
+          next_day
         end
 
         def min_time
@@ -186,9 +203,13 @@ module Groonga
 
         def least_over_time
           if @max_day.nil?
-            Time.local(@year, @month + 1, 1)
+            if @month == 12
+              Time.local(@year + 1, 1, 1)
+            else
+              Time.local(@year, @month + 1, 1)
+            end
           else
-            Time.local(@year, @month, @max_day + 1)
+            Time.local(@year, @month, @max_day)
           end
         end
 
