@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2; coding: utf-8 -*- */
 /*
-  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+  Copyright (C) 2008-2014  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -13,11 +13,11 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <hash.h>
-#include <pat.h>
+#include <grn_hash.h>
+#include <grn_pat.h>
 
 #include <gcutter.h>
 
@@ -182,7 +182,7 @@ test_hash(gconstpointer test_data)
   gchar *path;
   const gchar *test_case_names[] = {"test_stress_hash", NULL};
 
-  grn_test_assert(grn_ctx_init(context, GRN_CTX_USE_QL));
+  grn_test_assert(grn_ctx_init(context, 0));
 
   path = g_build_filename(base_dir, "hash", NULL);
   g_setenv(GRN_TEST_ENV_HASH_PATH, path, TRUE);
@@ -214,7 +214,7 @@ test_patricia_trie(gconstpointer test_data)
   gchar *path;
   const gchar *test_case_names[] = {"test_stress_patricia_trie", NULL};
 
-  grn_test_assert(grn_ctx_init(context, GRN_CTX_USE_QL));
+  grn_test_assert(grn_ctx_init(context, 0));
 
   path = g_build_filename(base_dir, "patricia-trie", NULL);
   g_setenv(GRN_TEST_ENV_PATRICIA_TRIE_PATH, path, TRUE);
